@@ -5,6 +5,7 @@
 # Written by Dan Allen <dan@mojavelinux.com>
 # - small additions by Stefan Kamphausen
 # - better completion code by Robi Malik robi.malik@nexgo.de
+# - trailing path support by Damon Harper <ds+dev-cdargs@usrbin.ca> Feb 2006
 
 # Globals
 CDARGS_SORT=0   # set to 1 if you want mark to sort the list
@@ -23,20 +24,42 @@ CDARGS_NODUPS=1 # set to 1 if you want mark to delete dups
 # @param  string alias                          #
 #                                               #
 # @access private                               #
-# @return void                                  #
+# @return 0 on success, >0 on failure           #
 # --------------------------------------------- #
 function _cdargs_get_dir ()
 {
-    # if there is one exact match, then just use
-    # that match without calling cdargs
-    if [[ -e "$HOME/.cdargs" && `grep -c "^$1 " "$HOME/.cdargs"` == 1 ]]; then
-        dir=`grep "^$1 " "$HOME/.cdargs" | sed "s/[^ ]* \(.*\)/\1/"`;
-    # okay, we need cdargs to resolve this one, 
-    # if we had a substring param, then force program window to come up
-    else
-        cdargs --noresolve "$1" && \
-        dir=`cat "$HOME/.cdargsresult"` && \
-        rm -f "$HOME/.cdargsresult";
+    local bookmark extrapath
+    # if there is one exact match (possibly with extra path info after it),
+    # then just use that match without calling cdargs
+    if [ -e "$HOME/.cdargs" ]; then
+        dir=`grep "^$1 " "$HOME/.cdargs"`
+        if [ -z "$dir" ]; then
+            bookmark="${1/\/*/}"
+            if [ "$bookmark" != "$1" ]; then
+                dir=`grep "^$bookmark " "$HOME/.cdargs"`
+                extrapath=`echo "$1" | sed 's#^[^/]*/#/#'`
+            fi
+        fi
+        [ -n "$dir" ] && dir=`echo "$dir" | sed 's/^[^ ]* //'`
+    fi
+    if [ -z "$dir" -o "$dir" != "${dir/
+/}" ]; then
+        # okay, we need cdargs to resolve this one.
+        # note: intentionally retain any extra path to add back to selection.
+        dir=
+        if cdargs --noresolve "${1/\/*/}"; then
+            dir=`cat "$HOME/.cdargsresult"`
+            rm -f "$HOME/.cdargsresult";
+        fi
+    fi
+    if [ -z "$dir" ]; then
+        echo "Aborted: no directory selected" >&2
+        return 1
+    fi
+    [ -n "$extrapath" ] && dir="$dir$extrapath"
+    if [ ! -d "$dir" ]; then
+        echo "Failed: no such directory '$dir'" >&2
+        return 2
     fi
 }
 
@@ -60,19 +83,20 @@ function _cdargs_exec ()
     # curses window will come up at which point a directory
     # will need to be choosen.  After selecting a directory,
     # the function will continue and $_cdargs_dir will be set.
-	if [ -e $last ]; then
-	  last=""
+    if [ -e $last ]; then
+        last=
+    fi
+    if _cdargs_get_dir "$last"; then
+        # For each argument save the last, move the file given in
+        # the argument to the resolved cdargs directory
+        i=1;
+        for arg; do
+            if [ $i -lt $# ]; then
+                $command "$arg" "$dir";
+            fi
+            let i=$i+1;
+        done
     fi
-    _cdargs_get_dir "$last";
-    # For each argument save the last, move the file given in
-    # the argument to the resolved cdargs directory
-    i=1;
-    for arg; do
-        if [ $i -lt $# ]; then
-            $command "$arg" "$dir";
-        fi
-        let i=$i+1;
-    done
 }
 
 # --------------------------------------------- #
@@ -188,14 +212,48 @@ function ca ()
 # --------------------------------------------- #
 function _cdargs_aliases ()
 {
-    cur=${COMP_WORDS[COMP_CWORD]}
+    local cur bookmark dir strip oldIFS
     COMPREPLY=()
-    COMPREPLY=( $( (echo $cur ; cat "$HOME/.cdargs") | \
-                   awk 'BEGIN {first=1}
-		        {if (first) {cur=$0; l=length(cur); first=0}
-                         else if (substr($1,1,l) == cur) {print $1}}' ) )
-
+    if [ -e "$HOME/.cdargs" ]; then
+        cur=${COMP_WORDS[COMP_CWORD]}
+        if [ "$cur" != "${cur/\//}" ]; then # if at least one /
+            bookmark="${cur/\/*/}"
+            dir=`grep "^$bookmark " "$HOME/.cdargs" | sed 's#^[^ ]* ##'`
+            if [ -n "$dir" -a "$dir" = "${dir/
+/}" -a -d "$dir" ]; then
+                strip="${dir//?/.}"
+                oldIFS="$IFS"
+                IFS='
+'
+                COMPREPLY=( $(
+                    compgen -d "$dir`echo "$cur" | sed 's#^[^/]*##'`" \
+                        | sed -e "s/^$strip/$bookmark/" -e "s/\([^\/a-zA-Z0-9#%_+\\\\,.-]\)/\\\\\\1/g" ) )
+                IFS="$oldIFS"
+            fi
+        else
+            COMPREPLY=( $( (echo $cur ; cat "$HOME/.cdargs") | \
+                           awk 'BEGIN {first=1}
+                                 {if (first) {cur=$0; l=length(cur); first=0}
+                                 else if (substr($1,1,l) == cur) {print $1}}' ) )
+        fi
+    fi
     return 0
 }
 
-complete -F _cdargs_aliases cv cb cdb
+# --------------------------------------------- #
+# Bash programming completion for cdargs        #
+# Set up completion (put in a function just so  #
+# `nospace' can be a local variable)            #
+#                                               #
+# @param  none                                  #
+#                                               #
+# @access private                               #
+# @return void                                  #
+# --------------------------------------------- #
+_cdargs_complete() {
+  local nospace=
+  [ "${BASH_VERSINFO[0]}" -ge 3 -o \( "${BASH_VERSINFO[0]}" = 2 -a \( "${BASH_VERSINFO[1]}" = 05a -o "${BASH_VERSINFO[1]}" = 05b \) \) ] && nospace='-o nospace'
+  complete $nospace -S / -X '*/' -F _cdargs_aliases cv cb cdb
+}
+
+_cdargs_complete
