@@ -5,7 +5,7 @@
  *     Copyright (C) 2001-2003 by Stefan Kamphausen
  *     Author: Stefan Kamphausen <http://www.skamphausen.de>
  *
- *     Time-stamp: <09-Oct-2005 12:41:36 ska>
+ *     Time-stamp: <26-Feb-2006 18:06:47 ska>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -582,10 +582,12 @@ list_from_file(void) {
         return false;
     }
     string cwd = get_cwd_as_string();
-    char linebuf[1024];
-    while (listfile.getline(linebuf,1000)) {
-        string line(linebuf);
-        // comments ... are not saved later so we simply don't allow them
+    while (! listfile.eof()) {
+        string line;
+        getline(listfile, line);
+        if(line == "") continue;
+        // comments ... are not saved later so we simply don't allow
+        // them
 //       if (line[0] == '#') {
 //          continue;
 //       }
@@ -910,7 +912,7 @@ void edit_list_file(void) {
     }
     endwin();
     list_to_file();
-    system((string(editor) + " " + get_listfile()).c_str());
+    system((string(editor) + " \"" + get_listfile() + "\"").c_str());
     list_from_file();
     refresh();
     init_curses();
